import { existsSync, promises } from 'fs';
import { resolve } from 'pathe';
import consola from 'consola';

async function buildModule(rootDir) {
  const { build } = await import('unbuild');
  await build(rootDir, false, {
    declaration: true,
    entries: [
      "src/module",
      { input: "src/runtime/", outDir: "dist/runtime", ext: "mjs" }
    ],
    rollup: {
      emitCJS: false,
      cjsBridge: true
    },
    externals: [
      "@nuxt/schema",
      "@nuxt/schema-edge",
      "@nuxt/kit",
      "@nuxt/kit-edge"
    ],
    hooks: {
      async "rollup:done"(ctx) {
        await writeCJSStub(ctx.options.outDir);
        const moduleEntryPath = resolve(ctx.options.outDir, "module.mjs");
        const moduleFn = await import(moduleEntryPath).then((r) => r.default || r).catch((err) => {
          consola.error(err);
          consola.error("Cannot load module. Please check dist:", moduleEntryPath);
          return null;
        });
        if (!moduleFn) {
          return;
        }
        const moduleMeta = await moduleFn.getMeta();
        await writeTypes(ctx.options.outDir, moduleMeta);
      }
    }
  });
}
async function writeTypes(distDir, meta) {
  const dtsFile = resolve(distDir, "types.d.ts");
  if (existsSync(dtsFile)) {
    return;
  }
  const schemaShims = [];
  if (meta.configKey) {
    schemaShims.push(`  interface NuxtConfig { ['${meta.configKey}']?: Partial<ModuleOptions> }`, `  interface NuxtOptions { ['${meta.configKey}']?: ModuleOptions }`);
  }
  const dtsContents = `
import type { ModuleOptions } from './module'

${schemaShims.length ? `declare module '@nuxt/schema' {
${schemaShims.join("\n")}
}
` : ""}
export * from './module'
`;
  await promises.writeFile(dtsFile, dtsContents, "utf8");
}
async function writeCJSStub(distDir) {
  const cjsStubFile = resolve(distDir, "module.cjs");
  if (existsSync(cjsStubFile)) {
    return;
  }
  const cjsStub = `module.exports = function(...args) {
  return import('./module.mjs').then(m => m.default.call(this, ...args))
}
module.exports.meta = require('../package.json')
`;
  await promises.writeFile(cjsStubFile, cjsStub, "utf8");
}

export { buildModule as b };
